diff --git a/lisp/net/rcirc.el b/lisp/net/rcirc.el
index d1f54c9..3df94a2 100644
--- a/lisp/net/rcirc.el
+++ b/lisp/net/rcirc.el
@@ -315,6 +315,12 @@ and the cdr part is used for encoding."
   :type 'function
   :group 'rcirc)
 
+(defcustom rcirc-kill-channel-buffers-with-server-buffer nil
+  "When non-nil, kills any channel buffers associated with a
+server buffer when the latter is killed."
+  :type 'boolean
+  :group 'rcirc)
+
 (defvar rcirc-nick nil)
 
 (defvar rcirc-prompt-start-marker nil)
@@ -981,9 +987,17 @@ If ALL is non-nil, update prompts in all IRC buffers."
            (eq (aref target 0) ?&))))
 
 (defun rcirc-kill-buffer-hook ()
-  "Part the channel when killing an rcirc buffer."
+  "Part the channel when killing an rcirc buffer.
+
+If rcirc-kill-channel-buffers-with-server-buffer is non-nil and
+the killed buffer is a server buffer, kills all of the channel
+buffers associated with it."
   (when (eq major-mode 'rcirc-mode)
-    (rcirc-clean-up-buffer "Killed buffer")))
+    (rcirc-clean-up-buffer "Killed buffer")
+    (when (and rcirc-buffer-alist ;; it's a server buffer
+               rcirc-kill-channel-buffers-with-server-buffer
+      (dolist (channel rcirc-buffer-alist)
+        (kill-buffer (cdr channel)))))))
 
 (defun rcirc-change-major-mode-hook ()
   "Part the channel when changing the major-mode."
